package owg.lgm;

import java.io.ByteArrayOutputStream;
import owg.lgm.util.FILE;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.HashMap;

import com.jogamp.opengl.GL2ES1;

import owg.lgm.gl.BMFont;
import owg.lgm.gl.BMFontRenderer;
import owg.lgm.plaf.PNGIO;
import owg.util.fileextensions.FileExtensions;

public class Fonts
{
	protected static final HashMap<String, BMFontRenderer> registry = new HashMap<>();
	
	public static void loadDefaultFont(GL2ES1 gl) throws IOException
	{
		BMFont font;
		try (InputStream in = Fonts.class.getResourceAsStream("default.fnt"))
		{
			byte[] buf = new byte[64];
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			int r;
			while((r = in.read(buf)) != -1)
				bos.write(buf, 0, r);
			font = BMFont.read(ByteBuffer.wrap(bos.toByteArray()).order(ByteOrder.LITTLE_ENDIAN));
		}
		
		IImage[] pages = new IImage[font.common.pages];
		for(int i = 0; i<pages.length; i++)
		{
			try(InputStream in = Fonts.class.getResourceAsStream(font.pages[i]))
			{
				pages[i] = PNGIO.load(in, font.pages[i]);
			}
		}
		BMFontRenderer defFont = new BMFontRenderer(gl, null, font, pages);
		registry.put(null, defFont);
	}
	
	public static void load(GL2ES1 gl, FILE current) throws IOException
	{
		String ext = FileExtensions.getFileExtension(current.getPath());
		if(!"fnt".equals(ext))
			return;
		String name = FileExtensions.stripFileExtension(current.getName());
		//System.out.println("Loading Font: "+current);
		BMFont font = BMFont.read(current.getBytes());
		FILE dir = current.getParentFile();
		IImage[] pages = new IImage[font.common.pages];
		for(int i = 0; i<pages.length; i++)
			pages[i] = PNGIO.load(dir.getChild(font.pages[i]));
		
		BMFontRenderer ren = new BMFontRenderer(gl, name, font, pages);
		if(registry.put(name, ren) != null)
			System.err.println("Font name collision: "+name);
	}
	
	public static BMFontRenderer get(String name)
	{
		BMFontRenderer r = registry.get(name);
		if(r == null)
			throw new ResourceException("No such font: "+name);
		return r;
	}
}
