package owg.lgm.ar1.accessories;

import static owg.lgm.plaf.Engine.EN;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;
import java.util.HashMap;

import com.jogamp.opengl.GL2ES1;

import owg.lgm.Activity;
import owg.lgm.Background;
import owg.lgm.Fonts;
import owg.lgm.Sprite;
import owg.lgm.gl.AttribArray;
import owg.lgm.gl.BMFontRenderer;
import owg.lgm.gl.BufferUsage;
import owg.lgm.gl.DataType;
import owg.lgm.gl.DrawCommand;
import owg.lgm.gl.IndexBufferObject;
import owg.lgm.gl.Mesh;
import owg.lgm.gl.PrimitiveType;
import owg.lgm.gl.Tex2D;
import owg.lgm.gl.VertexAttribute;
import owg.lgm.gl.VertexBufferObjectF;
import owg.lgm.in.MB;
import owg.lgm.in.MouseEventType;
import owg.lgm.in.VK;
import owg.lgm.room.ActivityChangeException;
import owg.lgm.room.ActivityChangeType;
import owg.lgm.room.Room;
import owg.lgm.util.Letterbox;
import owg.util.math.IntDiv;
import owg.util.math.Matrix4x4F;
import owg.util.math.Modulo;

import static owg.lgm.GMLib.*;

public abstract class AR1Dialog implements Activity
{
	protected static final String[] OK_OPTION = {"OK"};
	protected static final String[] YES_NO_OPTION = {"Yes", "No"};
	
	protected enum Mode
	{
		HORIZONTAL, VERTICAL, STRING;
	}

	public static class ConverseException extends RuntimeException
	{
		public final String[] options;
		public final Mode mode;

		public ConverseException(String message, Mode mode, String... options)
		{
			super(message);
			this.mode = mode;
			this.options = (options.length==0 ? OK_OPTION : options);
		}
	}
	public static class SleepException extends RuntimeException {}

	protected final HashMap<String, Object> responses = new HashMap<>();
	protected String currentMessage;
	protected String[] currentOptions;
	protected Mode currentMode;

	protected StringBuilder stringInput = new StringBuilder();
	protected int selection;
	protected int kbPressed = -1;
	protected int mouseHover = -1;
	protected int mousePressed = -1;

	int repeatL = 0;
	int repeatR = 0;
	int repeatU = 0;
	int repeatD = 0;
	int repeatE = 0;

	boolean releasedL = false;
	boolean releasedR = false;
	boolean releasedU = false;
	boolean releasedD = false;
	boolean releasedE = false;

	protected int sleepPtr;
	protected long sleepTime = 0;
	protected int sleepCount = 0;

	Background uiMessageBox = Background.get("messagbacc");
	Sprite uiButton = Sprite.get("butonmess");
	Sprite uiStipple = Sprite.get("stipplemess");
	{ uiStipple.setTextureRepeat(true); }
	BMFontRenderer font = Fonts.get(null);
	int messageColor = merge_color(c_lime, c_yellow, 0.5);
	int selectionColor = merge_color(c_white, c_blue, 0.25);
	int optionColor = merge_color(c_white, c_blue, 0.5);

	VertexBufferObjectF boxVBO;
	IndexBufferObject boxIBO;
	Mesh boxMesh, stippleMesh;

	public AR1Dialog() {}

	@Override
	public void start(GL2ES1 gl, ActivityChangeType type)
	{
		int vertCount = 16;
		FloatBuffer verts = ByteBuffer.allocateDirect(4*4*vertCount).order(ByteOrder.nativeOrder()).asFloatBuffer();
		boxVBO = new VertexBufferObjectF(gl, verts, BufferUsage.DYNAMIC_DRAW);

		int indCount = 3*2*3*3;
		ShortBuffer inds = ByteBuffer.allocateDirect(2*indCount).order(ByteOrder.nativeOrder()).asShortBuffer();
		for(int y = 0; y<3; y++)
		{
			for(int x = 0; x<3; x++)
			{
				inds.put((short)((y  )*4+x  )).put((short)((y+1)*4+x  )).put((short)((y  )*4+x+1));
				inds.put((short)((y  )*4+x+1)).put((short)((y+1)*4+x  )).put((short)((y+1)*4+x+1));
			}
		}

		inds.flip();
		boxIBO = new IndexBufferObject(gl, inds, BufferUsage.STATIC_DRAW);

		boxMesh = new Mesh(DrawCommand.drawElements(boxIBO, PrimitiveType.TRIANGLES, indCount, DataType.UNSIGNED_SHORT, 0), 
				new AttribArray(boxVBO, VertexAttribute.POSITION, 2, DataType.FLOAT, 4*4, 0), 
				new AttribArray(boxVBO, VertexAttribute.TEXTURE_COORDINATE_0, 2, DataType.FLOAT, 4*4, 2*4));
		boxMesh.autoDestroy(boxIBO);
		boxMesh.autoDestroy(boxVBO);

		FloatBuffer stippleVerts = ByteBuffer.allocateDirect(4*2*4).order(ByteOrder.nativeOrder()).asFloatBuffer();
		stippleVerts.put(1.0f).put(0.0f);
		stippleVerts.put(0.0f).put(0.0f);
		stippleVerts.put(0.0f).put(1.0f);
		stippleVerts.put(1.0f).put(1.0f);
		stippleVerts.flip();
		VertexBufferObjectF stippleVBO = new VertexBufferObjectF(EN.gl, stippleVerts, BufferUsage.STATIC_DRAW);
		stippleMesh = new Mesh(DrawCommand.drawArrays(PrimitiveType.LINE_LOOP, 0, 4), 
				new AttribArray(stippleVBO, VertexAttribute.POSITION, 2, DataType.FLOAT, 4*2, 0), 
				new AttribArray(stippleVBO, VertexAttribute.TEXTURE_COORDINATE_0, 2, DataType.FLOAT, 4*2, 0));
		stippleMesh.autoDestroy(stippleVBO);
	}

	@Override
	public void run(GL2ES1 gl)
	{	
		try {
			sleepPtr = 0;
			try {
				dialog();
			} catch(ActivityChangeException e) {
				//Need to kill current room now
				Room.CURRENT.end(gl, e.type);
				throw e;
			} 
			throw new ActivityChangeException(ActivityChangeType.PAUSE_RESUME, Room.CURRENT);
		} catch(ConverseException e) {
			if(currentMessage==null ? e.getMessage()!=null : !currentMessage.equals(e.getMessage())) {
				stringInput.setLength(0);
				stringInput.append(e.options[0]);
				selection = 0;
				kbPressed = -1;
				mouseHover = -1;
				mousePressed = -1;
			}
			currentMessage = e.getMessage();
			currentOptions = e.options;
			currentMode = e.mode;

			step();
			render();
		} catch(SleepException e) {
			currentMessage = null;
			currentOptions = null;
			currentMode = null;
			
			render();
		}
	}

	protected int[] viewportSize = new int[2];
	protected float[] view = new float[4];

	protected void step()
	{
		double mx = EN.in.getMouseX();
		double my = EN.in.getMouseY();

		Room.CURRENT.getViewportSize(viewportSize, 0);
		Letterbox.getViewport(view, 0, EN.getSurfaceWidth(), EN.getSurfaceHeight(), viewportSize[0], viewportSize[1]);
		mx = (mx-view[0])/view[2];
		my = (my-view[1])/view[3];

		mx *= 640;
		my *= 480;

		draw_set_font(font);

		mouseHover = -1;

		if(currentMode == Mode.STRING)
		{
			CharSequence chars = EN.in.getTypedChars();
			draw_set_font(font);
			for(int i = 0; i<chars.length(); i++)
			{
				char ch = chars.charAt(i);
				if(ch == '\b')
					stringInput.setLength(Math.max(0, stringInput.length()-1));
				else if(ch == '\n' || ch == '\r') {
					confirm();
					break;
				} else if(font.chars.containsKey(ch)) {
					stringInput.append(ch);
					if(string_width(stringInput) >= 112)
						stringInput.setLength(stringInput.length()-1);
				}
			}
		}
		else if(currentOptions != null)
		{
			int ow = getOptionWidth();
			int oh = getOptionHeight();
			
			for(int i = 0; i<currentOptions.length; i++)
			{
				int ox = getOptionX(i);
				int oy = getOptionY(i);
				if(Math.abs(mx-ox) <= ow/2 && my > oy && my < oy+oh)
					mouseHover = i;
			}

			for(int i = 0; i<EN.in.getMouseEvents().size(); i++)
			{
				MouseEventType type = EN.in.getMouseEvents().types.get(i);
				MB mb = EN.in.getMouseEvents().ids.get(i);
				if(mb == MB.LEFT)
				{
					if(type == MouseEventType.PRESS && mouseHover != -1 && kbPressed == -1)
					{	
						selection = mouseHover;
						mousePressed = mouseHover;
					}
					else if(type == MouseEventType.RELEASE && mousePressed != -1)
					{	
						if(mousePressed == mouseHover)
							confirm();
						mousePressed = -1;
					}
				}
			}
	
			if(mousePressed == -1)
			{
				if(keyboard_check(VK.LEFT)) {
					if(releasedL && currentMode==Mode.HORIZONTAL && trigger(++repeatL))
						selection = Modulo.mod(selection-1, currentOptions.length);
				} else {
					releasedL = true;
					repeatL = 0;
				}
				if(keyboard_check(VK.RIGHT)) {
					if(releasedR && currentMode==Mode.HORIZONTAL && trigger(++repeatR))
						selection = Modulo.mod(selection+1, currentOptions.length);
				} else {
					releasedR = true;
					repeatR = 0;
				}
				if(keyboard_check(VK.UP)) {
					if(releasedU && currentMode==Mode.VERTICAL && trigger(++repeatU))
						selection = Modulo.mod(selection-1, currentOptions.length);
				} else {
					releasedU = true;
					repeatU = 0;
				}
				if(keyboard_check(VK.DOWN)) {
					if(releasedD && currentMode==Mode.VERTICAL && trigger(++repeatD))
						selection = Modulo.mod(selection+1, currentOptions.length);
				} else {
					releasedD = true;
					repeatD = 0;
				}
				if(keyboard_check(VK.ENTER)) {
					if(releasedE && trigger(++repeatE))
						confirm();
				} else {
					releasedE = true;
					repeatE = 0;
				}
	
				if(keyboard_check_pressed(VK.SPACE) && kbPressed == -1)
					kbPressed = selection;
				if(keyboard_check_released(VK.SPACE) && kbPressed != -1)
				{
					if(kbPressed == selection)
						confirm();
					kbPressed = -1;
				}
			}
		}
	}

	protected void confirm()
	{
		if(currentMode == Mode.STRING)
			responses.put(currentMessage, stringInput.toString());
		else
			responses.put(currentMessage, selection);
	}

	protected boolean trigger(int repeater) {
		return repeater == 1 || (repeater > 20 && repeater%2==1);
	}

	protected void render()
	{
		Room.CURRENT.render();
		if(EN.fbm.beginRender(EN.getSurfaceWidth(), EN.getSurfaceHeight())) {
			EN.gl.glViewport(0, 0, 640, 480);
			EN.gl.glMatrixMode(GL2ES1.GL_PROJECTION);
			EN.gl.glLoadIdentity();
			EN.gl.glOrthof(0, 640, 0, 480, -32768, 32767);
			EN.gl.glMatrixMode(GL2ES1.GL_MODELVIEW);
			EN.gl.glLoadIdentity();

			renderDialog();
			EN.fbm.endRender();
		}
		EN.clear();
		EN.fbm.blit(EN.getSurfaceWidth(), EN.getSurfaceHeight());
	}

	protected void renderDialog() 
	{
		draw_set_font(font);
		draw_set_color(messageColor);
		draw_set_halign(fa_center);
		draw_set_valign(fa_bottom);

		if(currentMessage != null)
		{
			int baseY = currentMode==Mode.VERTICAL ? 96 : 224;
	
			int w = string_width_ext(currentMessage, -1, 512);
			int h = string_height_ext(currentMessage, -1, 512);
			renderBox(uiMessageBox.tex, false, 320-w/2-16, baseY-h-16, w+32, h+32);
	
			draw_text_ext(320, baseY, currentMessage, -1, 512);
		}

		if(currentMode == Mode.STRING)
		{
			renderBox(uiMessageBox.tex, false, 320-64, 265, 128, 32);
			draw_set_color(selectionColor);
			draw_set_halign(fa_left);
			draw_set_valign(fa_middle);
			boolean bar = EN.time%40L < 20L;
			if(bar)
				stringInput.append((char)166);
			draw_text(320-56, 280, stringInput);
			if(bar)
				stringInput.setLength(stringInput.length()-1);
		}
		else if(currentOptions != null)
		{
			draw_set_valign(fa_middle);
	
			int ow = getOptionWidth();
			int oh = getOptionHeight();
			for(int i = 0; i<currentOptions.length; i++)
			{
				int ox = getOptionX(i);
				int oy = getOptionY(i);
	
				int ii;
				if((mousePressed == i && mouseHover == i) || (kbPressed == i && selection == i)) {
					ii = 2;
					draw_set_color(optionColor);
				} else if(selection == i) {
					ii = 1;
					draw_set_color(selectionColor);
				} else {
					ii = 0;
					draw_set_color(optionColor);
				} 
				renderBox(uiButton.textures[ii], false, ox-ow/2, oy, ow, oh);
	
				if(selection == i)
					renderStipple(uiStipple.textures[ii], ox-ow/2+3.5f, oy+3.5f, ow-7, oh-7);
	
				draw_text(ox, oy+oh/2, currentOptions[i]);
			}
		}
		draw_set_halign(fa_left);
		draw_set_valign(fa_top);
	}

	protected int getOptionWidth()
	{
		int ow = 0;
		for(int i = 0; i<currentOptions.length; i++)
			ow = Math.max(ow, string_width(currentOptions[i]));
		return ow+32;
	}

	protected int getOptionHeight()
	{
		return 48;
	}

	protected int getOptionX(int option)
	{
		if(currentMode==Mode.VERTICAL)
			return 320;
		else
			return 320 + (160 * (-currentOptions.length+1 + 2*option))/2;
	}

	protected int getOptionY(int option)
	{
		if(currentMode==Mode.VERTICAL)
			return 128 + option * 48;
		else
			return 256;
	}

	protected float[] tm = Matrix4x4F.newIdentityMatrix4x4F();

	protected void renderStipple(Tex2D tex, float x0, float y0, float w, float h)
	{
		tex.begin();
		EN.gl.glColor4f(1, 1, 1, 1);
		EN.gl.glPushMatrix();
		EN.gl.glTranslatef(x0, y0, 0);
		EN.gl.glScalef(w, h, 0);
		EN.gl.glMatrixMode(GL2ES1.GL_TEXTURE);
		tm[0] = w/tex.texWidth;
		tm[4] = h/tex.texWidth;
		EN.gl.glLoadMatrixf(tm, 0);
		stippleMesh.render(EN.gl);
		EN.gl.glLoadIdentity();
		EN.gl.glMatrixMode(GL2ES1.GL_MODELVIEW);
		EN.gl.glPopMatrix();
		tex.end();
	}

	protected void renderBox(Tex2D tex, boolean invert, int x0, int y0, int w, int h)
	{
		FloatBuffer verts = boxVBO.getData();

		float x1, y1, x2, y2, x3, y3;
		x1 = x0 + tex.dataWidth/2f;
		y1 = y0 + tex.dataHeight/2f;

		x3 = x0+w;
		y3 = y0+h;
		x2 = x3 - tex.dataWidth/2f;
		y2 = y3 - tex.dataHeight/2f;

		float a,b;
		if(invert) {
			a = 1.0f; b = 0.0f;
		} else {
			a = 0.0f; b = 1.0f;
		}

		verts.put(x0).put(y0).put(a).put(a);
		verts.put(x1).put(y0).put(0.5f).put(a);
		verts.put(x2).put(y0).put(0.5f).put(a);
		verts.put(x3).put(y0).put(b).put(a);

		verts.put(x0).put(y1).put(a).put(0.5f);
		verts.put(x1).put(y1).put(0.5f).put(0.5f);
		verts.put(x2).put(y1).put(0.5f).put(0.5f);
		verts.put(x3).put(y1).put(b).put(0.5f);

		verts.put(x0).put(y2).put(a).put(0.5f);
		verts.put(x1).put(y2).put(0.5f).put(0.5f);
		verts.put(x2).put(y2).put(0.5f).put(0.5f);
		verts.put(x3).put(y2).put(b).put(0.5f);

		verts.put(x0).put(y3).put(a).put(b);
		verts.put(x1).put(y3).put(0.5f).put(b);
		verts.put(x2).put(y3).put(0.5f).put(b);
		verts.put(x3).put(y3).put(b).put(b);

		verts.flip();

		boxVBO.update();
		EN.gl.glColor4f(1, 1, 1, 1);
		tex.begin();
		boxMesh.render(EN.gl);
		tex.end();
	}

	@Override
	public void end(GL2ES1 gl, ActivityChangeType type)
	{
		boxMesh.dispose();
		stippleMesh.dispose();
	}

	public abstract void dialog() throws ConverseException, SleepException;

	public void show_message(String message) throws ConverseException
	{
		if(!responses.containsKey(message))
			throw new ConverseException(message, Mode.HORIZONTAL);
	}
	public boolean show_question(String message) throws ConverseException
	{
		Integer r = (Integer) responses.get(message);
		if(r == null)
			throw new ConverseException(message, Mode.HORIZONTAL, YES_NO_OPTION);
		return r == 0;
	}
	public int show_message_ext(String message, String... options) throws ConverseException
	{
		Integer r = (Integer) responses.get(message);
		if(r == null)
			throw new ConverseException(message, Mode.HORIZONTAL, options);
		return r;
	}

	public int show_menu(String message, String... options) throws ConverseException
	{
		Integer r = (Integer) responses.get(message);
		if(r == null)
			throw new ConverseException(message, Mode.VERTICAL, options);
		return r;
	}
	
	public String get_string(String message, String def)
	{
		String r = (String) responses.get(message);
		if(r == null)
			throw new ConverseException(message, Mode.STRING, def);
		return r;
	}
	
	public void sleep(int millis) throws SleepException
	{
		if(sleepPtr == sleepCount)
		{
			long nanos = millis * 1_000_000;
			if(sleepTime < nanos)
			{
				sleepTime += IntDiv.roundDiv(1_000_000_000, EN.animator.getTargetFPS());
				throw new SleepException();
			}
			else
			{
				sleepCount++;
				sleepTime = 0;
			}
		}
		sleepPtr++;
	}

	public void start() throws ActivityChangeException
	{
		throw new ActivityChangeException(ActivityChangeType.PAUSE_RESUME, this);
	}

	public static void simple_message(String string)
	{
		new AR1Dialog()
		{
			@Override
			public void dialog() throws ConverseException
			{
				show_message(string);
			}
		}.start();
	}
}
