package owg.lgm.gl;

import com.jogamp.opengl.GL2ES1;

public enum VertexAttribute
{
	POSITION() { //0
		@Override
		public void enable(GL2ES1 gl, AttribArray ptr) {
			gl.glEnableClientState(GL2ES1.GL_VERTEX_ARRAY);
			gl.glVertexPointer(ptr.elementCount, ptr.dataType.value, ptr.byteStride, ptr.byteOffset);
		}
		@Override
		public void disable(GL2ES1 gl, AttribArray ptr) {
			gl.glDisableClientState(GL2ES1.GL_VERTEX_ARRAY);
		}
	}, 				
	NORMAL() { //1
		@Override
		public void enable(GL2ES1 gl, AttribArray ptr) {
			gl.glEnableClientState(GL2ES1.GL_VERTEX_ARRAY);
			assert ptr.elementCount == 3;
			gl.glNormalPointer(ptr.dataType.value, ptr.byteStride, ptr.byteOffset);
		}
		@Override
		public void disable(GL2ES1 gl, AttribArray ptr) {
			gl.glDisableClientState(GL2ES1.GL_VERTEX_ARRAY);
		}
	},						
	COLOR() { //2
		@Override
		public void enable(GL2ES1 gl, AttribArray ptr) {
			gl.glEnableClientState(GL2ES1.GL_COLOR_ARRAY);
			gl.glColorPointer(ptr.elementCount, ptr.dataType.value, ptr.byteStride, ptr.byteOffset);
		}
		@Override
		public void disable(GL2ES1 gl, AttribArray ptr) {
			gl.glDisableClientState(GL2ES1.GL_COLOR_ARRAY);
		}
	},						
	TEXTURE_COORDINATE_0() {  //3
		@Override
		public void enable(GL2ES1 gl, AttribArray ptr) {
			gl.glClientActiveTexture(GL2ES1.GL_TEXTURE0);
			gl.glEnableClientState(GL2ES1.GL_TEXTURE_COORD_ARRAY);
			gl.glTexCoordPointer(ptr.elementCount, ptr.dataType.value, ptr.byteStride, ptr.byteOffset);
		}
		@Override
		public void disable(GL2ES1 gl, AttribArray ptr) {
			gl.glClientActiveTexture(GL2ES1.GL_TEXTURE0);
			gl.glDisableClientState(GL2ES1.GL_TEXTURE_COORD_ARRAY);
		}
	},
	TEXTURE_COORDINATE_1() {  //4
		@Override
		public void enable(GL2ES1 gl, AttribArray ptr) {
			gl.glClientActiveTexture(GL2ES1.GL_TEXTURE0);
			gl.glEnableClientState(GL2ES1.GL_TEXTURE_COORD_ARRAY);
			gl.glTexCoordPointer(ptr.elementCount, ptr.dataType.value, ptr.byteStride, ptr.byteOffset);
		}
		@Override
		public void disable(GL2ES1 gl, AttribArray ptr) {
			gl.glClientActiveTexture(GL2ES1.GL_TEXTURE0);
			gl.glDisableClientState(GL2ES1.GL_TEXTURE_COORD_ARRAY);
		}
	},
	TEXTURE_COORDINATE_2() {  //5
		@Override
		public void enable(GL2ES1 gl, AttribArray ptr) {
			gl.glClientActiveTexture(GL2ES1.GL_TEXTURE0);
			gl.glEnableClientState(GL2ES1.GL_TEXTURE_COORD_ARRAY);
			gl.glTexCoordPointer(ptr.elementCount, ptr.dataType.value, ptr.byteStride, ptr.byteOffset);
		}
		@Override
		public void disable(GL2ES1 gl, AttribArray ptr) {
			gl.glClientActiveTexture(GL2ES1.GL_TEXTURE1);
			gl.glDisableClientState(GL2ES1.GL_TEXTURE_COORD_ARRAY);
		}
	},
	TEXTURE_COORDINATE_3() {  //6
		@Override
		public void enable(GL2ES1 gl, AttribArray ptr) {
			gl.glClientActiveTexture(GL2ES1.GL_TEXTURE2);
			gl.glEnableClientState(GL2ES1.GL_TEXTURE_COORD_ARRAY);
			gl.glTexCoordPointer(ptr.elementCount, ptr.dataType.value, ptr.byteStride, ptr.byteOffset);
		}
		@Override
		public void disable(GL2ES1 gl, AttribArray ptr) {
			gl.glClientActiveTexture(GL2ES1.GL_TEXTURE3);
			gl.glDisableClientState(GL2ES1.GL_TEXTURE_COORD_ARRAY);
		}
	};
	
	public abstract void enable(GL2ES1 gl, AttribArray ptr);
	public abstract void disable(GL2ES1 gl, AttribArray ptr);
}
