package owg.lgm.gl.fbo;

import com.jogamp.opengl.GL2ES1;
import com.jogamp.opengl.GLException;

import owg.lgm.gl.GLResource;

public class FramebufferObject extends GLResource
{
	public Attachment[] attachments;

	protected transient GL2ES1 gl;
	protected transient int handle;
	
	public FramebufferObject(GL2ES1 gl, Attachment... attachments)
	{
		this.attachments = attachments;
		init(gl);
	}
	
	protected FramebufferObject() {}
	
	public int getHandle()
	{
		return handle;
	}

	@Override
	public int getInitOrder() {
		return 1;
	}

	@Override
	public void init(GL2ES1 gl)
	{
		this.gl = gl;
		{
			int[] f = new int[1];
			gl.glGenFramebuffers(1, f, 0);
			handle = f[0];
		}
		applyAttachments();
	}
	
	public void setAttachments(Attachment... attachments)
	{
		this.attachments = attachments;
		applyAttachments();
	}
	
	protected void applyAttachments()
	{
		if(attachments.length > 0)
		{
			gl.glBindFramebuffer(GL2ES1.GL_FRAMEBUFFER, handle);
			for(int i = 0; i<attachments.length; i++)
				attachments[i].attach(gl);
			checkStatus();
			gl.glBindFramebuffer(GL2ES1.GL_FRAMEBUFFER, 0);
		}
	}
	
	public void checkStatus()
	{
		int status = gl.glCheckFramebufferStatus(GL2ES1.GL_FRAMEBUFFER);
        switch(status)
        {
        	case GL2ES1.GL_FRAMEBUFFER_COMPLETE:
        		break;
        	case GL2ES1.GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT:
                throw new GLException("GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT: Deleted, zero-size, or non-renderable renderable attachment");
        	case GL2ES1.GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS:
                throw new GLException("GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS: Different renderable sizes");
        	case GL2ES1.GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT:
                throw new GLException("GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT: No renderable attachments");
        	case GL2ES1.GL_FRAMEBUFFER_UNSUPPORTED:
                throw new GLException("GL_FRAMEBUFFER_UNSUPPORTED: Internal formats of attachments incompatible with driver");
            default:
                throw new GLException("Unknown framebuffer completeness error : "+status+"(0x"+Integer.toHexString(status)+")");
        }
	}

	@Override
	public void uninit()
	{
		gl.glDeleteFramebuffers(1, new int[] {handle}, 0);
		handle = 0;
	}

	public void bind()
	{
		gl.glBindFramebuffer(GL2ES1.GL_FRAMEBUFFER, handle);
	}
	
	public void unbind()
	{
		gl.glBindFramebuffer(GL2ES1.GL_FRAMEBUFFER, 0);
	}
}
