package owg.lgm.room;

import static owg.lgm.plaf.Engine.EN;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map.Entry;

import owg.lgm.room.RoomData.TileInfo;
import owg.lgm.util.AutoDestructibleImpl;
import owg.util.primitivelists.IntList;

public class TileManager extends AutoDestructibleImpl
{
	protected final ArrayList<TileLayer> layers = new ArrayList<>(32);
	protected int idGen = 10000000;

	public boolean setVisible(int id, boolean b)
	{
		for(int i = 0; i<layers.size(); i++)
		{
			if(layers.get(i).setVisible(id, b))
				return true;
		}
		return false;
	}

	public boolean remove(int id)
	{
		for(int i = 0; i<layers.size(); i++)
		{
			TileLayer l = layers.get(i);
			if(l.remove(id))
			{
				if(l.isEmpty())
					layers.remove(i);
				return true;
			}
		}
		return false;
	}

	public void add(TileInfo tile)
	{
		TileLayer layer = null;
		search: {
			for(int i = 0; i<layers.size(); i++)
			{
				TileLayer l = layers.get(i);
				if(layers.get(i).depth == tile.depth)
				{
					layer = l;
					break search;
				}
			}
			//Not found
			layer = new TileLayer();
			layer.init(EN.gl, tile.depth, Collections.emptyList());
			autoDestroy(layer);
			layers.add(layer);
		}
		tile.id = idGen;
		idGen++;
		layer.add(EN.gl, tile);
	}

	public void getTiles(IntList output)
	{
		for(int i = 0; i<layers.size(); i++)
			layers.get(i).getTiles(output);
	}

	public boolean getTileInfo(TileInfo output, int id)
	{
		for(int i = 0; i<layers.size(); i++)
		{
			if(layers.get(i).getTileInfo(output, id))
				return true;
		}
		return false;
	}

	public void init(TileInfo[] tileInfo)
	{
		HashMap<Integer, ArrayList<TileInfo>> tileMap = new HashMap<>();
		for(int i = 0; i<tileInfo.length; i++)
		{
			TileInfo tile = tileInfo[i];
			idGen = Math.max(idGen, tile.id+1);
			ArrayList<TileInfo> list = tileMap.get(tile.depth);
			if(list == null)
			{
				list = new ArrayList<>();
				tileMap.put(tile.depth, list);
			}
			list.add(tile);
		}

		for(Entry<Integer, ArrayList<TileInfo>> e : tileMap.entrySet())
		{
			TileLayer layer = new TileLayer();
			autoDestroy(layer);
			layer.init(EN.gl, e.getKey(), e.getValue());
			layers.add(layer);
		}
	}
}
