package owg.lgm.util;

import static owg.lgm.plaf.Engine.EN;

import java.nio.ByteBuffer;

import com.jogamp.opengl.GL2ES1;

import owg.lgm.IImage;
import owg.lgm.IImage.Type;
import owg.lgm.gl.Mesh;
import owg.lgm.gl.Tex2D;
import owg.lgm.gl.TexFormat;
import owg.lgm.gl.TexSampler;
import owg.util.math.PowerOfTwo;

public class FramebufferManagerDummy extends AutoDestructibleImpl implements FramebufferManager
{
	protected GL2ES1 gl;
	protected Mesh nwRect;
	protected Tex2D rbo;
	protected int viewportWidth, viewportHeight;
	protected float[] view = new float[4];
	protected boolean valid;
	
	public FramebufferManagerDummy(Mesh nwRect)
	{
		this.nwRect = nwRect;
	}
	
	@Override
	public GL2ES1 getGL()
	{
		return gl;
	}

	@Override
	public void configure(GL2ES1 gl, int viewportWidth, int viewportHeight)
	{
		this.gl = gl;
		this.viewportWidth = viewportWidth;
		this.viewportHeight = viewportHeight;
		
		if(rbo != null && (rbo.texWidth < viewportWidth || rbo.texHeight < viewportHeight))
		{
			autoDestructibles.remove(rbo);
			rbo.dispose();
			rbo = null;
		}
		if(rbo == null)
		{
			int w = PowerOfTwo.nextPowerOfTwo(viewportWidth);
			int h = PowerOfTwo.nextPowerOfTwo(viewportHeight);
			rbo = new Tex2D(gl, TexFormat.RGBA, null, w, h, false, TexSampler.CLAMP);
			autoDestroy(rbo);
		}
	}

	@Override
	public boolean beginRender(int surfaceWidth, int surfaceHeight)
	{
		return (valid = surfaceWidth >= viewportWidth && surfaceHeight >= viewportHeight);
	}

	@Override
	public void endRender()
	{
		rbo.begin();
		gl.glCopyTexSubImage2D(GL2ES1.GL_TEXTURE_2D, 0, 0, 0, 0, 0, viewportWidth, viewportHeight);
		rbo.end();
	}

	@Override
	public void blit(int surfaceWidth, int surfaceHeight)
	{
		if(!valid)
			return;
		gl.glViewport(0, 0, surfaceWidth, surfaceHeight);
		
		gl.glMatrixMode(GL2ES1.GL_PROJECTION);
		gl.glLoadIdentity();
		gl.glOrthof(0, surfaceWidth, surfaceHeight, 0, -1, 1);
		gl.glMatrixMode(GL2ES1.GL_MODELVIEW);
		gl.glLoadIdentity();
		Letterbox.getViewport(view, 0, surfaceWidth, surfaceHeight, viewportWidth, viewportHeight);
		gl.glTranslatef(view[0], view[1], 0);
		gl.glScalef(view[2], view[3], 1);
		
		gl.glDisable(GL2ES1.GL_BLEND);
		gl.glMatrixMode(GL2ES1.GL_TEXTURE);
		gl.glScalef((float)viewportWidth/rbo.texWidth, (float)viewportHeight/rbo.texHeight, 1);
		gl.glColor4f(1, 1, 1, 1);
		rbo.begin();
		nwRect.render(gl);
		rbo.end();
		gl.glLoadIdentity();
		gl.glMatrixMode(GL2ES1.GL_MODELVIEW);
		gl.glLoadIdentity();
		gl.glEnable(GL2ES1.GL_BLEND);
	}

	@Override
	public IImage screenshot(int x0, int y0, int w, int h)
	{
		IImage image = new IImage(Type.RGBA, w, h, null);
		//ES1 cannot read back texture data
		//work around by drawing the texture to the screen, then reading the screen pixels
		if(valid)
		{
			int[] rect = new int[4];
			gl.glGetIntegerv(GL2ES1.GL_VIEWPORT, rect, 0);
			gl.glViewport(0, 0, viewportWidth, viewportHeight);
			gl.glDisable(GL2ES1.GL_BLEND);
			
			gl.glPushMatrix();
			gl.glLoadIdentity();
			gl.glTranslatef(-1, -1, 0);
			gl.glScalef(2, 2, 1);
			gl.glMatrixMode(GL2ES1.GL_PROJECTION);
			gl.glPushMatrix();
			gl.glLoadIdentity();
			
			gl.glMatrixMode(GL2ES1.GL_TEXTURE);
			gl.glScalef((float)viewportWidth/rbo.texWidth, (float)viewportHeight/rbo.texHeight, 1);
			gl.glColor4f(1, 1, 1, 1);
			rbo.begin();
			EN.nwRect.render(gl);
			rbo.end();
			gl.glLoadIdentity();
			gl.glMatrixMode(GL2ES1.GL_PROJECTION);
			gl.glPopMatrix();
			gl.glMatrixMode(GL2ES1.GL_MODELVIEW);
			gl.glPopMatrix();
			gl.glEnable(GL2ES1.GL_BLEND);
			
			gl.glReadPixels(x0, y0, w, h, GL2ES1.GL_RGBA, GL2ES1.GL_UNSIGNED_BYTE, ByteBuffer.wrap(image.data));
			gl.glViewport(rect[0], rect[1], rect[2], rect[3]);
			gl.glEnable(GL2ES1.GL_BLEND);
		}
		image.flip();
		return image;
	}

	@Override
	public IImage screenshot()
	{
		return screenshot(0, 0, viewportWidth, viewportHeight);
	}

	@Override
	public int getCurrentWidth()
	{
		return viewportWidth;
	}

	@Override
	public int getCurrentHeight()
	{
		return viewportHeight;
	}

}
