package owg.lgm.util;

import java.nio.ByteBuffer;
import com.jogamp.opengl.GL2ES1;

import owg.lgm.IImage;
import owg.lgm.IImage.Type;
import owg.lgm.gl.Mesh;
import owg.lgm.gl.RenderBuffer;
import owg.lgm.gl.RenderBufferFormat;
import owg.lgm.gl.Tex2D;
import owg.lgm.gl.TexFormat;
import owg.lgm.gl.TexSampler;
import owg.lgm.gl.fbo.Attachment;
import owg.lgm.gl.fbo.AttachmentPoint;
import owg.lgm.gl.fbo.FramebufferObject;
import owg.lgm.gl.fbo.RenderBufferAttachment;
import owg.lgm.gl.fbo.Tex2DAttachment;

public class FramebufferManagerImpl extends AutoDestructibleImpl implements FramebufferManager
{
	protected GL2ES1 gl;
	protected Mesh nwRect;
	protected Tex2D rbo = null;
	protected RenderBuffer dbo = null;
	protected FramebufferObject fbo = null;
	protected int viewportWidth, viewportHeight;
	protected float[] view = new float[4];
	protected boolean valid;
	protected final boolean depthBuffer;

	public FramebufferManagerImpl(Mesh nwRect, boolean depthBuffer)
	{
		this.nwRect = nwRect;
		this.depthBuffer = depthBuffer;
	}

	@Override
	public GL2ES1 getGL()
	{
		return gl;
	}

	@Override
	public void configure(GL2ES1 gl, int viewportWidth, int viewportHeight)
	{
		this.gl = gl;
		if(fbo == null)
		{
			fbo = new FramebufferObject(gl);
			autoDestroy(fbo);
		}
		this.viewportWidth = viewportWidth;
		this.viewportHeight = viewportHeight;
		valid = (viewportWidth > 0 && viewportHeight > 0);
		if(valid)
		{
			if(rbo == null || rbo.texWidth < viewportWidth || rbo.texHeight < viewportHeight)
			{
				if(rbo != null) {
					autoDestructibles.remove(rbo);
					rbo.dispose();
				}
				rbo = new Tex2D(gl, TexFormat.RGBA, null, viewportWidth, viewportHeight, false, TexSampler.CLAMP);
				autoDestroy(rbo);
				
				Attachment[] attachments;
				
				if(depthBuffer)
				{
					if(dbo != null)
					{
						autoDestructibles.remove(dbo);
						dbo.dispose();
					}
					dbo = new RenderBuffer(gl, RenderBufferFormat.DEPTH16, viewportWidth, viewportHeight);
					attachments = new Attachment[] { 
							new Tex2DAttachment(AttachmentPoint.COLOR_ATTACHMENT0, rbo),
							new RenderBufferAttachment(AttachmentPoint.DEPTH_ATTACHMENT, dbo)
					};
				}
				else
					attachments = new Attachment[] { new Tex2DAttachment(AttachmentPoint.COLOR_ATTACHMENT0, rbo) };
				
				fbo.setAttachments(attachments);
			}
		}
	}

	@Override
	public boolean beginRender(int surfaceWidth, int surfaceHeight)
	{
		if(!valid)
			return false;
		fbo.bind();
		return true;
	}
	
	@Override
	public void endRender()
	{
		fbo.unbind();
	}

	@Override
	public void blit(int surfaceWidth, int surfaceHeight)
	{
		if(!valid)
			return;
		gl.glViewport(0, 0, surfaceWidth, surfaceHeight);
		
		gl.glMatrixMode(GL2ES1.GL_PROJECTION);
		gl.glLoadIdentity();
		gl.glOrthof(0, surfaceWidth, surfaceHeight, 0, -1, 1);
		gl.glMatrixMode(GL2ES1.GL_MODELVIEW);
		gl.glLoadIdentity();
		Letterbox.getViewport(view, 0, surfaceWidth, surfaceHeight, viewportWidth, viewportHeight);
		gl.glTranslatef(view[0], view[1], 0);
		gl.glScalef(view[2], view[3], 1);
		
		gl.glDisable(GL2ES1.GL_BLEND);
		gl.glMatrixMode(GL2ES1.GL_TEXTURE);
		gl.glScalef((float)viewportWidth/rbo.texWidth, (float)viewportHeight/rbo.texHeight, 1);
		gl.glColor4f(1, 1, 1, 1);
		rbo.begin();
		nwRect.render(gl);
		rbo.end();
		gl.glLoadIdentity();
		gl.glMatrixMode(GL2ES1.GL_MODELVIEW);
		gl.glLoadIdentity();
		gl.glEnable(GL2ES1.GL_BLEND);
	}

	@Override
	public IImage screenshot(int x0, int y0, int w, int h)
	{
		IImage image = new IImage(Type.RGBA, w, h, null);
		if(valid)
		{
			fbo.bind();
			gl.glReadPixels(x0, y0, w, h, GL2ES1.GL_RGBA, GL2ES1.GL_UNSIGNED_BYTE, ByteBuffer.wrap(image.data));
			fbo.unbind();
		}
		//image.flip();
		return image;
	}

	@Override
	public IImage screenshot()
	{
		return screenshot(0, 0, viewportWidth, viewportHeight);
	}

	@Override
	public int getCurrentWidth()
	{
		return viewportWidth;
	}
	
	@Override
	public int getCurrentHeight()
	{
		return viewportHeight;
	}
}
