package owg.competent;

public enum SoundModel {
	/**<p>
	 *The mono sound model performs mixing in a single channel, 
	 *and outputs sound equally to all speakers.
	 *</p><p>
	 *When using the mono sound model, channel vectors have only one element,
	 *which indicates the gain for that sample channel.
	 *</p><p>
	 *The mono sound model has the lowest memory and CPU usage footprint, 
	 *and may be used by applications that do not need any form of positional audio.
	 *</p>*/
	MONO(1, 1, 1),
	/**<p>
	 *The stereo sound model performs mixing in two channels: Left and right.
	 *The left channel is output to the left speaker(s) and vice versa.
	 *Center speakers receive a weighted average of the two channels.
	 *</p><p>
	 *When using the stereo sound model, channel vectors have two elements,
	 *indicating the left and right gain multipliers for that sample channel.
	 *</p><p>
	 *The stereo sound model is appropriate for most applications.
	 *</p>*/
	STEREO(2, 2, 2);
	
	public final int id;
	public final int frameSize;
	public final int channelVectorSize;
	private SoundModel(int id, int frameSize, int channelVectorSize) {
		this.id = id;
		this.frameSize = frameSize;
		this.channelVectorSize = channelVectorSize;
	}
}
