package owg.competent.soundfile;

import java.io.IOException;
import java.io.InputStream;

import owg.competent.SampleFormat;

public class WavInputStream extends SizedAudioDecoderStream 
{
	protected WAV wav;
	protected SampleFormat sampleFormat;
	protected int readBytes = 0;
	
	public WavInputStream(InputStream in) throws IOException 
	{
		super(in);
		wav = WAV.prepare(in);
		if(wav.fmt.encoding == WAV.PCM_FLOAT)
		{
			if(wav.fmt.bitDepth == 32)
				sampleFormat = SampleFormat.FLOAT32;
			else
				throw new IOException("Unsupported floating point format bit depth: "+wav.fmt.bitDepth);
		}
		else if(wav.fmt.encoding == WAV.PCM_INT)
		{
			if(wav.fmt.bitDepth == 32)
				sampleFormat = SampleFormat.INT32;
			else if(wav.fmt.bitDepth == 24)
				sampleFormat = SampleFormat.INT24;
			else if(wav.fmt.bitDepth == 16)
					sampleFormat = SampleFormat.INT16;
			else if(wav.fmt.bitDepth == 8)
				sampleFormat = SampleFormat.UINT8;
			else
				throw new IOException("Unsupported integer format bit depth: "+wav.fmt.bitDepth);			
		}
		else
			throw new IOException("Unsupported WAV encoding: "+wav.fmt.encoding);
	}

	@Override
	public double getSampleRate() 
	{
		return wav.fmt.frameRate;
	}

	@Override
	public int getChannelCount() 
	{
		return wav.fmt.channels;
	}

	@Override
	public SampleFormat getSampleFormat() {
		return sampleFormat;
	}

	@Override
	public boolean isLittleEndian() {
		return true;
	}
	
	@Override
	public int getSize() {
		return wav.dataSize;
	}
	
    @Override
	public int read() throws IOException {
    	if(available() <= 0)
    		return -1;
    	else {
    		int r = in.read();
    		if(r>=0)
    			readBytes++;
    		return r;
    	}
    }
    //Note: super.read(byte[]) is sane
    @Override
	public int read(byte b[], int off, int len) throws IOException {
    	if(available() <= 0)
    		return -1;
    	else
    		return increment(in.read(b, off, Math.min(len, available())));
    }
    protected int increment(int r)
    {
    	if(r > 0)
    		readBytes+=r;
    	return r;
    }
    @Override
	public long skip(long n) throws IOException {
        return increment((int)in.skip(Math.min(n, available())));
    }
    @Override
	public int available() throws IOException {
		return wav.dataSize-readBytes;
    }
}
